/***************************************************************************
**+----------------------------------------------------------------------+**
**|                                                                      |**
**|              Copyright (c) 2006 ACORP-Merlion.                       |**
**|              Copyright (c) 2006 Mokrushin I.V. aka McMCC             |**
**|                        ALL RIGHTS RESERVED                           |**
**|                                                                      |**
**+----------------------------------------------------------------------+**
***************************************************************************/

#include "_tistdtypes1.h"
#include "pal_defs.h"
#include "DDC_switch.h"
#include "CSL_switchPrvt.h"
#ifdef LINUX_COMPILATION
#include "linuxpal.h"
#else
#ifdef VX_COMPILATION
#include "vxWorkspal.h"
#endif
#endif

#define RTL_8305SC_LINK_SHIFT_MASK     2
#define RTL_8305SC_LINK_MASK           0x1

#define RTL_8305SC_DUPLEX_SHIFT_MASK   8
#define RTL_8305SC_DUPLEX_MASK         0x1

#define RTL_8305SC_SPEED_SHIFT_MASK    13
#define RTL_8305SC_SPEED_MASK          0x1

#define RTL_8305SC_AUTO_ENABLE_SHIFT_MASK      12
#define RTL_8305SC_AUTO_ENABLE_MASK            0x1

#define RTL_8305SC_AUTO_RESTART_SHIFT_MASK     9
#define RTL_8305SC_AUTO_RESTART_MASK           0x1

/*******************************************************************************
*
* cslRtl8305scStart - 

*
*/
Int32 cslRtl8305scStart (PCSL_SWITCH_DRV_HANDLE swHead)
{
    Uint32 i;
    
    swHead->numberOfPorts = 5; /* init number ports */
    /* Set Auto Negotiation */
    for(i = 0; i < swHead->numberOfPorts; i++)
	phyRegWrite(swHead, 0, i,
	            (Int32)(RTL_8305SC_AUTO_ENABLE_MASK << RTL_8305SC_AUTO_ENABLE_SHIFT_MASK) |
		    (Int32)(RTL_8305SC_AUTO_RESTART_MASK << RTL_8305SC_AUTO_RESTART_SHIFT_MASK)); 
    return PAL_True;
}

/*******************************************************************************
*
* cslRtl8305scRegVal2OperStatus - extract the user operational status out of register value.
*
* This code contains switch specific information. The main purpose is converting switch specific info to 
* the neutral code. As a neutral data types we're using external interfaces.
*
* INPUTS:
*     [In]    OPER_TYPE oper 
*     [In]    Uint16   data
* 
*
*
* RETURNS: 
*       operational status to be returned to user
*
*
*/

Int32 cslRtl8305scRegVal2OperStatus (OPER_TYPE_E oper, Uint16 data)
{
    Int32 retVal = -1;
    
    if (oper == OPER_LINK)
    {
        retVal = (Int32)data & (Int32)(RTL_8305SC_LINK_MASK << RTL_8305SC_LINK_SHIFT_MASK);
        if(retVal == 0)
            return PORT_LINK_DOWN;
        return PORT_LINK_UP;
    }

    else if (oper == OPER_DUPLEX)
    {
        retVal = (Int32)data & (Int32)(RTL_8305SC_DUPLEX_MASK << RTL_8305SC_DUPLEX_SHIFT_MASK);
        if(retVal == 0)
            return PORT_DUPLEX_HALF;
        return PORT_DUPLEX_FULL;
    }

    else if (oper == OPER_SPEED)
    {
        retVal = (Int32)data & (Int32)(RTL_8305SC_SPEED_MASK << RTL_8305SC_SPEED_SHIFT_MASK);
        if(retVal == 0)
            return PORT_SPEED_10;
        return PORT_SPEED_100;
    }

    else if (oper == OPER_AUTO)
    {
        retVal = (Int32)data & (Int32)(RTL_8305SC_AUTO_ENABLE_MASK << RTL_8305SC_AUTO_ENABLE_SHIFT_MASK);
        if (retVal == 0)
            return PORT_AUTO_NO;
        return PORT_AUTO_YES;
    }
    
    return retVal;
}

/*******************************************************************************
*
* cslRtl8305scOperStatus2RegVal - constructs the register value to be written out of data and user request
*
* Enable/Disable a selected port
*
* INPUTS:
*     [In]    OPER_TYPE     oper 
*     [In]    Int32        operValue
*     [In]    Uint16       data
* 
*
*
* RETURNS: 
*       The final register value
*
*
*/

Uint16 cslRtl8305scOperStatus2RegVal (OPER_TYPE_E oper,Int32 operValue,Uint16 data)
{
    Uint16 retVal = 0xFFFF;
    
    if (oper == OPER_DUPLEX_SET)
    {
    	retVal = data & (~((Uint16)(RTL_8305SC_DUPLEX_MASK << RTL_8305SC_DUPLEX_SHIFT_MASK)));
        if(operValue == PORT_DUPLEX_FULL)
            retVal |= (Uint16)(RTL_8305SC_DUPLEX_MASK << RTL_8305SC_DUPLEX_SHIFT_MASK);
	retVal |= (Uint16)(RTL_8305SC_AUTO_RESTART_MASK << RTL_8305SC_AUTO_RESTART_SHIFT_MASK);
    }
    
    else if (oper == OPER_SPEED_SET)
    {
    	retVal = data & (~((Uint16)(RTL_8305SC_SPEED_MASK << RTL_8305SC_SPEED_SHIFT_MASK)));
        if(operValue == PORT_SPEED_100)
            retVal |= (Uint16)(RTL_8305SC_SPEED_MASK << RTL_8305SC_SPEED_SHIFT_MASK);
	retVal |= (Uint16)(RTL_8305SC_AUTO_RESTART_MASK << RTL_8305SC_AUTO_RESTART_SHIFT_MASK);
    }

    else if (oper == OPER_AUTO)
    {
    	retVal = data & (~((Uint16)(RTL_8305SC_AUTO_ENABLE_MASK << RTL_8305SC_AUTO_ENABLE_SHIFT_MASK)));
        if(operValue == PORT_AUTO_YES) {
            retVal |= (Uint16)(RTL_8305SC_AUTO_ENABLE_MASK << RTL_8305SC_AUTO_ENABLE_SHIFT_MASK);
	    retVal |= (Uint16)(RTL_8305SC_AUTO_RESTART_MASK << RTL_8305SC_AUTO_RESTART_SHIFT_MASK);
	}
        else if (operValue == PORT_AUTO_RESTART)
            retVal |= (Uint16)(RTL_8305SC_AUTO_RESTART_MASK << RTL_8305SC_AUTO_RESTART_SHIFT_MASK);
    }

    return retVal;
}


/*******************************************************************************
*
* cslRtl8305scSwitchGetVersion - returns the switch specific information
*
* 
*
* INPUTS:
*     [Out]   PSW_VER_INFO pSwitchInfo
* 
*
*
* RETURNS: 
*       Status
*
*
*/

void cslRtl8305scSwitchGetVersion(PCSL_SWITCH_DRV_HANDLE swDrv,PSW_VER_INFO pSwitchInfo)
{
    Uint32 regaddr,phyaddr;
    
#if 1
    /* See Datasheet RTL8305C-Ver.C page 56 */    
    regaddr = 0x3;
    phyaddr = 0;
    phyRegRead(swDrv, regaddr, phyaddr, pSwitchInfo->Model); /* return 0x05 RTL8305 */
    /* */
#else
    pSwitchInfo->Model                  = 5;
#endif
    pSwitchInfo->is802_1_P_capable      = PAL_True; 
    pSwitchInfo->is802_1_Q_capable      = PAL_True;
    pSwitchInfo->maxNumberOfPorts       = 5;
    pSwitchInfo->maxNumberOfPriorities  = 0;
    pSwitchInfo->maxNumberOfVLANs       = 0;
    
}

        
/*******************************************************************************
*
* cslRtl8305scPortNum2RegAddr - returns the register address
*
* Retreive register address from port and operator type.
*
* INPUTS:
*     [In]   OPER_TYPE_E    type
*     [In]   Uint32         portNum
*
*
* RETURNS: 
*       Register address
*
*
*/

Uint32 cslRtl8305scPortNum2RegAddr(OPER_TYPE_E type, Uint32 portNum)
{
    Uint32 regAddr = 0;

    switch (type) 
            {  /* TBD :restart Auto-Negotiation,Power down */
                case OPER_LINK: 
                    regAddr = 0x01; 
                    break; 
                case OPER_DUPLEX: 
                    regAddr = 0x0; 
                    break; 
                case OPER_SPEED: 
                    regAddr = 0x0; 
                    break; 
                case OPER_AUTO: 
                    regAddr = 0x0; 
                    break; 

                default: 
                   regAddr = 0xFFFF;
                    break; 
        } 

    return regAddr;
}

/*******************************************************************************
*
* cslRtl8305scPortNum2PHYAddr - returns the PHY address
*
* Retreive PHY address from port and operator type.
*
* INPUTS:
*     [In]   OPER_TYPE_E    type
*     [In]   Uint32         portNum
*
*
* RETURNS: 
*       Register address
*
*
*/


Uint32 cslRtl8305scPortNum2PHYAddr(OPER_TYPE_E type, Uint32 portNum)
{
    Uint32 phyAddr = 0;
    switch (type) 
        { 
            case OPER_LINK: 
                phyAddr = portNum-1; 
                break; 
            case OPER_DUPLEX_SET: 
            case OPER_DUPLEX: 
                phyAddr = portNum-1; 
                break; 
            case OPER_SPEED_SET: 
            case OPER_SPEED: 
                phyAddr = portNum-1;
                break; 
            case OPER_AUTO: 
                phyAddr = portNum-1; 
                break; 
            default: 
               phyAddr = 0xFFFF;
                break; 
        }

    return phyAddr;
}


/*******************************************************************************
*
* cslRtl8305scPrepareReadStatSequence -  Rtl8305sc doesn't 
*                                          support statistics counters
*
*/
Int32 cslRtl8305scPrepareReadStatSequence(PCSL_SWITCH_DRV_HANDLE swHead,Uint32 port)
{
    return PAL_True;
}

/*******************************************************************************
*
* cslRtl8305scGetPreparedCounters - Rtl8305sc doesn't 
*                                     support statistics counters
*
*/
Int32 cslRtl8305scGetPreparedCounters(PCSL_SWITCH_DRV_HANDLE swHead,Uint32 statIndex,Uint32* counter)
{    
        *counter  = 0;
        return PAL_True;
}
/*******************************************************************************
*
* cslRtl8305scSpecialCapsSet - Rtl8305sc Dosen't support VLAN
*                                     
*
*/
Int32 cslRtl8305scSpecialCapsSet(PCSL_SWITCH_DRV_HANDLE swHead, Uint32 cmd, Uint32 arg)
{
    return PAL_True;
}
