/*************************************************************************************** 
*  chief_busybox_patch1.c  (cobbled from uClibc 0.9.19 sources)
* 6 March 2006
* 
* Patch for BusyBox 1.x, which complains about isnan and isinf not being found
* 1. This file should go into the coreutils/ directory
* 2. coreutils/Makefile.in should then be amended as follows:
*    after the line listing "sort.o", 
*    COREUTILS-$(CONFIG_SORT)      += chief_busybox_patch1.o
*    
***************************************************************************************/
enum {
  FP_SNAN                       = 0,    /*      signaling NaN
      */
  FP_QNAN                       = 1,    /*      quiet NaN
      */
  FP_INFINITE                   = 2,    /*      + or - infinity
      */ 
  FP_ZERO                       = 3,    /*      + or - zero
      */
  FP_NORMAL                     = 4,    /*      all normal numbers 
      */
  FP_SUBNORMAL                  = 5     /*      denormal numbers 
      */
};     

typedef struct                   /*      Hex representation of a double.      */
      {
#if defined(__BIG_ENDIAN__)
      unsigned long int high;
      unsigned long int low;
#else
      unsigned long int low;
      unsigned long int high;
#endif
      } dHexParts;

typedef union
      {
      unsigned char byties[8];
      double dbl;
      } DblInHex;

#define      dExpMask           0x7FF00000
#define      dHighMan           0x000FFFFF
#define FEXP_MASK 0x7f800000
#define FFRAC_MASK 0x007fffff

/***********************************************************************
   long int __fpclassifyf(float x) returns the classification code of the
   argument x, as defined in <fp.h>.
   
   Exceptions:  INVALID signaled if x is a signaling NaN; in this case,
                the FP_QNAN code is returned.
   
   Calls:  none
***********************************************************************/

long int __fpclassifyf ( float x )
{
   unsigned long int iexp;
   
   union {
      unsigned long int lval;
      float fval;
   } z;
   
   z.fval = x;
   iexp = z.lval & FEXP_MASK;                 /* isolate float exponent */ 
   
   if (iexp == FEXP_MASK) {                   /* NaN or INF case */
      if ((z.lval & 0x007fffff) == 0)
         return (long int) FP_INFINITE;
      else if ((z.lval & 0x00400000) != 0)
         return (long int) FP_QNAN;
      else
         return (long int) FP_SNAN;
   }
   
   if (iexp != 0)                             /* normal float */
      return (long int) FP_NORMAL;
      
   if (x == 0.0)
      return (long int) FP_ZERO;             /* zero */
   else
      return (long int) FP_SUBNORMAL;        /* must be subnormal */
}
   

/***********************************************************************
      Function __fpclassify,                                                 
      Implementation of classify of a double number for the PowerPC.          
                                                                              
   Exceptions:  INVALID signaled if x is a signaling NaN; in this case,
                the FP_QNAN code is returned.
   
   Calls:  none
***********************************************************************/

long int __fpclassify ( double arg )
{
	register unsigned long int exponent;
      union
            {
            dHexParts hex;
            double dbl;
            } x;
      
	x.dbl = arg;
	
	exponent = x.hex.high & dExpMask;
	if ( exponent == dExpMask )
		{
		if ( ( ( x.hex.high & dHighMan ) | x.hex.low ) == 0 )
			return (long int) FP_INFINITE;
		else
            	return ( x.hex.high & 0x00080000 ) ? FP_QNAN : FP_SNAN; 
		}
	else if ( exponent != 0)
		return (long int) FP_NORMAL;
	else {
		if ( arg == 0.0 )
			return (long int) FP_ZERO;
		else
			return (long int) FP_SUBNORMAL;
		}
}


int __signbitf (float __x)
{
  __extension__ union { float __f; int __i; } __u = { __f: __x };
  return __u.__i < 0;
}

int __signbit (double __x)
{
  __extension__ union { double __d; int __i[2]; } __u = { __d: __x };
  return __u.__i[0] < 0;
}

/***********************************************************************
* long int __isinff(float x) returns -1 if value represents  negative
*	infinity,  1  if value represents positive infinity,
*	and 0 otherwise.
*
* Calls:  __signbit
* +***********************************************************************/
long int __isinff ( float x )
{
    long int class = __fpclassifyf(x);
    if ( class == FP_INFINITE ) {
	return ( (__signbitf(x)) ? -1 : 1);
    }
    return 0;
}
//weak_alias (__isinff, isinff)
//#define isinff __isinff

long int __isinf ( double x )
{
    long int class = __fpclassify(x);
    if ( class == FP_INFINITE ) {
	return ( (__signbit(x)) ? -1 : 1);
    }
    return 0;
}
//weak_alias (__isinf, isinf)
//#define isinf __isinf

#if 0
long int __isinfl ( long double x )
{
    long int class = __fpclassify(x);
    if ( class == FP_INFINITE ) {
	return ( (__signbit(x)) ? -1 : 1);
    }
    return 0;
}
weak_alias (__isinfl, isinfl);
#else
//weak_alias (__isinf, isinfl)
//#define isinfl __isinf
#endif

/***********************************************************************
   long int __isnanf(float x) returns nonzero if and only if x is a
   NaN and zero otherwise.
   
   Exceptions:  INVALID is raised if x is a signaling NaN; in this case,
                nonzero is returned.
   
   Calls:  none
***********************************************************************/

long int __isnanf ( float x )
{   
   union {
      unsigned long int lval;
      float fval;
   } z;
   
   z.fval = x;
   return (((z.lval&FEXP_MASK) == FEXP_MASK) && ((z.lval&FFRAC_MASK) != 0));
}
//weak_alias (__isnanf, isnanf);
//#define isnanf __isnanf

long int __isnan ( double x )
{
	long int class = __fpclassify(x);
	return ( ( class == FP_SNAN ) || ( class == FP_QNAN ) ); 
}
//weak_alias (__isnan, isnan);
//#define isnan __isnan

#if 0
long int __isnanl ( long double x )
{
	long int class = __fpclassify(x);
	return ( ( class == FP_SNAN ) || ( class == FP_QNAN ) ); 
}
weak_alias (__isnanl, isnanl);
#else
//weak_alias (__isnan, isnanl);
//#define isnanl __isnan
#endif

