/***************************************************************************\
*****************************************************************************
**                                                                         **
** source file:    client.c                                                **
** last revision:  28/03/2004                                              **
**                                                                         **
*****************************************************************************
*****************************************************************************
**                                                                         **
** This program is free software; you can redistribute it and/or modify it **
** under the terms of the GNU General Public License as published by the   **
** Free Software Foundation; either version 2, or (at your option) any     **
** later version.                                                          **
**                                                                         **
** This program is distributed in the hope that it will be useful,         **
** but WITHOUT ANY WARRANTY; without even the implied warranty of          **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           **
** GNU General Public License for more details.                            **
**                                                                         **
** You should have received a copy of the GNU General Public License       **
** along with this program; if not, write to the Free Software             **
** Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.                     **
**                                                                         **
*****************************************************************************
\***************************************************************************/

//--------------------------------------------------------------------------------------------
//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//--------------------------------------------------------------------------------------------

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "socket.h"
#include "error.h"
#include "file_io.h"
#include "common.h"
#include "utils.h"
#include "memory.h"
#include "client.h"

//--------------------------------------------------------------------------------------------
//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//--------------------------------------------------------------------------------------------

int start_client(int port, char *host, int nbr_files, struct file_t *file, MEM_MANAGER *mem_manager)
{
  int my_socket;                    // The Socket
  int i;                            // Storage For Loops And Temp Int
  int bytes_sent;                   // The Number Of Bytes Sent
  int bytes_left;                   // The Number Of Bytes Left
  size_t file_listing_size;         // The Size Of The File Listing
  struct sockaddr_in server;        // The Server SockAddr
  struct hostent *server_infos;     // The Server's Info (Ip - HostName)
  struct timeval time_current;      // The Current Time
  struct timeval time_last_update;  // The Time For The Next Update Of The Kbps Counter
  float speed;                      // The Transfer's Speed
  float new_speed;                  // The Next Transfer's Speed We'll Display
  char *dial_string;                // This Will Contain The Dialog Strings
  char *tmp_string;                 // Small String
  char *file_buffer;                // This Will Be A Storage For The File Buffer
  char *ptr_buffer;                 // This Is A Pointer To A Buffer

  memset(&server, 0, sizeof(struct sockaddr_in));
 
  /** We Allocate Some Memory **/
  if ((mem_alloc(mem_manager, (void **) &dial_string, 512)) != 0)
    return -1;

  if ((mem_alloc(mem_manager, (void **) &tmp_string, 64)) != 0)
    return -1;

  /** We Create A Socket **/
  if ((my_socket = create_socket()) < 0)
      return -1;

  /** We Try To Resolve The HostName **/
  server_infos = gethostbyname(host);

  if (server_infos == NULL)
    {
      error("gethostbyname", "unable to resolve the given hostname", 0);
      return -1;
    }

  sprintf(tmp_string, "%i.%i.%i.%i", (unsigned char) server_infos->h_addr_list[0][0], (unsigned char) server_infos->h_addr_list[0][1],\
                                     (unsigned char) server_infos->h_addr_list[0][2], (unsigned char) server_infos->h_addr_list[0][3]);

  /** We Get Everything Ready In Order To Connect To The Server **/
  if (inet_aton(tmp_string, &server.sin_addr) == 0)
    {
      error("inet_aton", NULL, EAFNOSUPPORT);
      return -1;
    }

  server.sin_family = AF_INET;
  server.sin_port = htons(port);

  /** We Connect The Client To The Server **/
  fprintf(stderr, "Connecting to server %s...\n", host);
  if (connect(my_socket, (struct sockaddr *) &server, sizeof(server)) != 0)
    {
      error("connect", NULL, errno);
      return -1;
    }

  fprintf(stderr, "Connection has been established.\n\n");

  /** Our Socket Must Not Block **/
  if ((fcntl(my_socket, F_SETFL, O_NONBLOCK)) < 0)
    {
      error("fcntl", NULL, errno);
      return stop_client(my_socket);
    }

  /** We Wait For Input From The Socket **/
  if ((wait_for_input(my_socket, TIME_OUT)) != 0)
    return stop_client(my_socket);

  /** We Print The Welcome Message **/
  if ((read_from_socket(my_socket, dial_string, mem_sizeof(mem_manager, dial_string))) < 0)
    return stop_client(my_socket);

  fprintf(stderr, "%s", dial_string);

  /** We Send To The Server The Size Of The Informations About The Files **/
  sprintf(tmp_string, "%d\n", nbr_files);
  file_listing_size = 1 + strlen(tmp_string);

  for (i=0; i<nbr_files; ++i)
    {
      file_listing_size += strlen(file[i].name);
      sprintf(tmp_string, "\n%d\n%ld\n", file[i].size, file[i].checksum);
      file_listing_size += strlen(tmp_string);
    }

  file_listing_size += strlen(STR_CONFIRM);

  sprintf(dial_string, "%d", file_listing_size);

  if ((write_to_socket(my_socket, dial_string, 1+strlen(dial_string))) < 0)
    return stop_client(my_socket);

  /** We Reallocate Some Memory If We Have To **/
  if ((mem_sizeof(mem_manager, dial_string)) < file_listing_size)
    {
      if ((mem_realloc(mem_manager, (void **) &dial_string, file_listing_size)) != 0)
        return stop_client(my_socket);
    }

  /** We Make The String **/
  sprintf(dial_string, "%d\n", nbr_files);
  
  for (i=0; i<nbr_files; ++i)
    {
      strcat(dial_string, file[i].name);
      sprintf(tmp_string, "\n%d\n%ld\n", file[i].size, file[i].checksum);
      strcat(dial_string, tmp_string);
    }

  strcat(dial_string, STR_CONFIRM);

  /** We Send The File Listing **/
  bytes_sent = 0;
  bytes_left = file_listing_size;
  ptr_buffer = dial_string;

  while (bytes_sent < file_listing_size)
    {
      if ((get_confirmation(my_socket, tmp_string, mem_sizeof(mem_manager, tmp_string))) != 0)
        return stop_client(my_socket);

      if ((bytes_left - DIAL_BUFFER_SIZE) < 0)  // We Are Near The End Of The String In This Case
        {
          if ((write_to_socket(my_socket, ptr_buffer, bytes_left)) < 0)
            return stop_client(my_socket);

          bytes_sent += bytes_left;
          ptr_buffer = dial_string;
          bytes_left = 0;
        }
      else                                     // Here, We Send A Few Bytes And Move The Pointer
        {
          if ((write_to_socket(my_socket, ptr_buffer, DIAL_BUFFER_SIZE)) < 0)
            return stop_client(my_socket);

          bytes_sent += DIAL_BUFFER_SIZE;
          ptr_buffer += DIAL_BUFFER_SIZE;
          bytes_left -= DIAL_BUFFER_SIZE;
        }
    }

  /** We Allocate Some Memory For The File Buffer **/
  if ((mem_alloc(mem_manager, (void **) &file_buffer, FILE_BUFFER_SIZE)) != 0)
    return stop_client(my_socket);

  /** We Send The Files **/
  for (i=0; (i<nbr_files); ++i)
    {
      int in_file; // File Handle

      bytes_sent = 0;
      bytes_left = 0;

      if ((in_file = open_file(file[i].full_name, O_RDONLY)) < 0)
        return stop_client(my_socket);

      if ((bytes_left = read_from_file(in_file, file_buffer, FILE_BUFFER_SIZE)) < 0)
        return stop_client(my_socket);

      ptr_buffer = file_buffer;
      gettimeofday(&time_current, NULL);
      time_last_update = time_current;
      speed = -1.0f;
      new_speed = 0.0f;

      fprintf(stderr, "\nSending %s 0%%", file[i].name);

      while (bytes_sent < file[i].size)
        {
          if ((get_confirmation(my_socket, tmp_string, mem_sizeof(mem_manager, tmp_string))) != 0)
            return stop_client(my_socket);

          if ((bytes_left - DIAL_BUFFER_SIZE) < 0) // We Are Near The End Of The File In This Case
            {
              if ((write_to_socket(my_socket, ptr_buffer, bytes_left)) < 0)
                return stop_client(my_socket);

              bytes_sent += bytes_left;
              ptr_buffer = file_buffer;
              bytes_left = 0;
            }
          else                                     // Here, We Send A Few Bytes And Re-Fill The Buffer If Needed
            {
              if ((write_to_socket(my_socket, ptr_buffer, DIAL_BUFFER_SIZE)) < 0)
                return stop_client(my_socket);

              bytes_sent += DIAL_BUFFER_SIZE;
              ptr_buffer += DIAL_BUFFER_SIZE;
              new_speed += (float) DIAL_BUFFER_SIZE;
              bytes_left -= DIAL_BUFFER_SIZE;

              if ((bytes_left == 0) && (bytes_sent < file[i].size))
                {
                  if ((bytes_left = read_from_file(in_file, file_buffer, FILE_BUFFER_SIZE)) < 0)
                    return stop_client(my_socket);

                  ptr_buffer = file_buffer;
                }		  
            }

          gettimeofday(&time_current, NULL);

          if (time_current.tv_sec > time_last_update.tv_sec)
            {
              float elapsed_time = (time_current.tv_sec - time_last_update.tv_sec) + ((time_current.tv_usec - time_last_update.tv_usec) / 1000000.0f);

              if (elapsed_time >= 2.0f)
                {
                  new_speed = ((new_speed  / elapsed_time) / 1000.0f);
                  speed = ((speed + new_speed) / 2.0f);
                  new_speed = 0.0f;
                  gettimeofday(&time_last_update, NULL);
                }
            }

          if (speed > 0.0f)
            fprintf(stderr, "\r\t                                                             \rSending %s %d%% (%.1f kbps)", file[i].name, (int) ((float) bytes_sent / (float) file[i].size * 100.0f), speed);
          else
            fprintf(stderr, "\r\t                                                             \rSending %s %d%% (-- kbps)", file[i].name, (int) ((float) bytes_sent / (float) file[i].size * 100.0f));
        }

      if ((close(in_file)) < 0)
        return stop_client(my_socket);

      fprintf(stderr, "\r\t                                                             \rSending %s 100%%", file[i].name);
    }

  fprintf(stderr, "\n\nAll the files have been sent successfully.\n\n");

  /** We Wait For The Bye Bye Message **/
  if ((wait_for_input(my_socket, TIME_OUT)) != 0)
    return stop_client(my_socket);

  if ((read_from_socket(my_socket, dial_string, mem_sizeof(mem_manager, dial_string))) < 0)
      return stop_client(my_socket);

  fprintf(stderr, "%s\n", dial_string);

  /** We Close The Socket **/
  if ((close_socket(my_socket)) < 0)
    return -1;

  return 0;
}

//--------------------------------------------------------------------------------------------
//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//--------------------------------------------------------------------------------------------

int stop_client(int socket)
{
  write_to_socket(socket, STR_STOP, 1+strlen(STR_STOP));
  shutdown(socket, SHUT_RDWR);
  close(socket);
  return -1;
}

//--------------------------------------------------------------------------------------------
//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//--------------------------------------------------------------------------------------------
