/***************************************************************************\
*****************************************************************************
**                                                                         **
** source file:    init.c                                                  **
** last revision:  28/03/2004                                              **
**                                                                         **
*****************************************************************************
*****************************************************************************
**                                                                         **
** This program is free software; you can redistribute it and/or modify it **
** under the terms of the GNU General Public License as published by the   **
** Free Software Foundation; either version 2, or (at your option) any     **
** later version.                                                          **
**                                                                         **
** This program is distributed in the hope that it will be useful,         **
** but WITHOUT ANY WARRANTY; without even the implied warranty of          **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           **
** GNU General Public License for more details.                            **
**                                                                         **
** You should have received a copy of the GNU General Public License       **
** along with this program; if not, write to the Free Software             **
** Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.                     **
**                                                                         **
*****************************************************************************
\***************************************************************************/

//---------------------------------------------------------------------------------------
//|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//---------------------------------------------------------------------------------------

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <errno.h>
#include "common.h"
#include "init.h"
#include "utils.h"
#include "memory.h"
#include "error.h"

//---------------------------------------------------------------------------------------
//|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//---------------------------------------------------------------------------------------

void usage(void)
{
  fprintf(stderr, "\n%s (Simple File Transfer) version %s\n\n", PROGRAM_NAME, PROGRAM_VERSION);
  fprintf(stderr, "usage: server: sft -s [OPTIONS]\nusage: client: sft -c <ip/hostname> [OPTIONS] <files>\n\n");
  fprintf(stderr, "General Options:\n\t"                                \
	          "-h\thelp\n\t"                                        \
	          "-v\tversion\n\n"                                     \
	          "Network Options:\n\t"                                \
	          "-p <n>\tport\n\n"                                    \
	          "Server Options:\n\t"                                 \
	          "-s\tserver mode\n\t"                                 \
                  "-k\tkeep the server running\n\t"                     \
	          "-d <p>\tchange reception directory\n\t"              \
	          "-l <n>\tmaximum amount of bytes we can receive\n\n"  \
	          "Client Options:\n\t"                                 \
	          "-c <ip>\tclient mode with server's IPv4 address\n\t" \
	          "-d <p>\tchange working directory\n\t"                \
	          "<files>\tfiles to send to the server\n\n");
  return;
}

//---------------------------------------------------------------------------------------
//|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//---------------------------------------------------------------------------------------

int init_arg(int argc, char *argv[], struct options_t *opt, MEM_MANAGER *mem_manager)
{
  char c;                 // Char For getopt()
  char *tmp;              // Pointer For Temp Values
  int i, j, k;            // Ints For Loops And Temp Values
  struct stat file_stat;  // Informations About Input Files

  memset(opt, 0, sizeof(struct options_t));

  /** We Read The Arguments That Have Been Sent To The Program **/
  while ((c = getopt(argc, argv, "hvskp:d:l:c:")) != EOF)
    {
      switch (c)
        {
        case 'h': // Help Requested
          {
            usage();
            return -1;
            break;
          }

        case 'v': // Version Requested
          {
            fprintf(stderr, "\n%s (Simple File Transfer) version %s\n\n", PROGRAM_NAME, PROGRAM_VERSION);
            fprintf(stderr, "Written by %s.\n\n", PROGRAM_AUTHORS);
            fprintf(stderr, "This program is free software; you can redistribute it\nand/or modify it under the terms \
of the GNU General Public License\nas published by the Free Software Foundation; either version 2,\nor (at your option) any later version.\n\n");
            return -1;
            break;
          }

        case 's': // Server Mode
          {
            if (opt->mode == 'c')
              {
                error("init_arg", "cannot be both server and client", 0);
                return -1;
              }

            opt->mode = 's';
            break;
          }

        case 'k': // Keep Running In Server Mode
          {
            opt->keep_running = 1;
            break;
          }

        case 'p': // Port
          {
            opt->port = atoi(optarg);
            break;
          }

        case 'd': // Working Directory
          {
            if ((mem_alloc(mem_manager, (void **) &opt->dir, 1+strlen(optarg))) != 0)
              return -1;

            strcpy(opt->dir, optarg);

            if ((chdir(opt->dir)) != 0)
              {
                error("chdir", NULL, errno);
                return -1;
              }

            break;
          }

        case 'l': // Size Limit
          {
            opt->limit = (long int) atol(optarg);
            break;
          }

        case 'c': // The Host Address
          {
            if (opt->mode == 's')
              {
                error("init_arg", "cannot be both server and client", 0);
                return -1;
              }

            opt->mode = 'c';

            if ((mem_alloc(mem_manager, (void **) &opt->address, 1+strlen(optarg))) != 0)
              return -1;

            strcpy(opt->address, optarg);
            break;
          }
        }
    }

  /** We Verify That At Least One Mode Has Been Choosen **/
  if ((opt->mode != 'c') && (opt->mode != 's'))
    {
      usage();
      return -1;
    }

  /** We Initialize Some Client Stuffs **/
  if (opt->mode == 'c')
    {
      opt->nbr_files = argc - optind;

      if (opt->nbr_files == 0)
        {
          usage();
          return -1;
        }

      // We Get Some Memory For The Files Informations
      if ((mem_alloc(mem_manager, (void **) &opt->file, (opt->nbr_files * sizeof(struct file_t)))) != 0)
        return -1;

      for (i=0; i<(opt->nbr_files); ++i)
        {
          // We Store The File Size
          if (stat(argv[optind+i], &file_stat) != 0)
            {
              error("stat", NULL, errno);
              return -1;
            }

          // We Verify That The File Is A File ;)
          if ((!S_ISREG(file_stat.st_mode)) && (!S_ISLNK(file_stat.st_mode)))
            {
              fprintf(stderr, "%s: error: %s is not a file nor a symbolic link.\n\n", PROGRAM_NAME, argv[optind+i]);
              return -1;
            }

          opt->file[i].size = file_stat.st_size;

          // We Store The File Checksum
          opt->file[i].checksum = get_file_checksum(argv[optind+i]);

          // We Store The Full File Name
          if ((mem_alloc(mem_manager, (void **) &opt->file[i].full_name, strlen(argv[optind+i])+1)) != 0)
            return -1;

          strcpy(opt->file[i].full_name, argv[optind+i]);

          // We Parse The Full File Name
          if ((tmp = strrchr(argv[optind+i], '/')) == NULL)  // We Look For A Path Delimiter
            {                                                // If We Can't Find Anyone, We Put The Current Path
              // We Get Some Memory For The File Name
              if ((mem_alloc(mem_manager, (void **) &opt->file[i].name, strlen(argv[optind+i])+1)) != 0)
                return -1;

              // We Get Some mem_manager, Memory For The Path
              if ((mem_alloc(mem_manager, (void **) &opt->file[i].path, 3)) != 0)
                return -1;

              strcpy(opt->file[i].name, argv[optind+i]);
              strcpy(opt->file[i].path, "./");
            }
          else                                               // If We Find One, We Copy The Good Stuffs In The Good Storages
            {
              ++tmp;
              j = 1 + strlen(argv[optind+i]); // Length Of The Whole String + \0
              k = tmp - argv[optind+i];       // Length Of The Path Part

              // We Get Some Memory For The File Name
              if ((mem_alloc(mem_manager, (void **) &opt->file[i].name, j-k+1)) != 0)
                return -1;

              // We Get Some Memory For The Path
              if ((mem_alloc(mem_manager, (void **) &opt->file[i].path, k+1)) != 0)
                return -1;

              memcpy(opt->file[i].name, tmp, (j - k));
              memcpy(opt->file[i].path, argv[optind+i], k);
              opt->file[i].path[k] = 0;
            }
        }
    }

  /** We Put The Default Values If None Have Been Changed **/
  if (opt->port == 0)  opt->port = 1984;

  return 0;
}

//---------------------------------------------------------------------------------------
//|||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//---------------------------------------------------------------------------------------
