/***************************************************************************\
*****************************************************************************
**                                                                         **
** source file:    utils.c                                                 **
** last revision:  18/12/2003                                              **
**                                                                         **
*****************************************************************************
*****************************************************************************
**                                                                         **
** This program is free software; you can redistribute it and/or modify it **
** under the terms of the GNU General Public License as published by the   **
** Free Software Foundation; either version 2, or (at your option) any     **
** later version.                                                          **
**                                                                         **
** This program is distributed in the hope that it will be useful,         **
** but WITHOUT ANY WARRANTY; without even the implied warranty of          **
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           **
** GNU General Public License for more details.                            **
**                                                                         **
** You should have received a copy of the GNU General Public License       **
** along with this program; if not, write to the Free Software             **
** Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.                     **
**                                                                         **
*****************************************************************************
\***************************************************************************/

//--------------------------------------------------------------------------------------------
//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//--------------------------------------------------------------------------------------------

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/utsname.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include "error.h"
#include "file_io.h"
#include "common.h"
#include "utils.h"

//--------------------------------------------------------------------------------------------
//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//--------------------------------------------------------------------------------------------

int get_sys_infos(char *infos)
{
  struct utsname uts_infos;

  if ((uname(&uts_infos)) != 0)
    {
      error("uname", NULL, errno);
      return -1;
    }

  sprintf(infos, "%s %s (%s)", uts_infos.sysname, uts_infos.release, uts_infos.machine);
  return 0;
}


//--------------------------------------------------------------------------------------------
//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//--------------------------------------------------------------------------------------------

long get_buf_checksum(char *buf, const size_t size)
{
  long checksum = 0;
  char *ptr;

  for (ptr=buf; ptr<(buf+size); ++ptr)
    checksum += *ptr;

  return checksum;
}


//--------------------------------------------------------------------------------------------
//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//--------------------------------------------------------------------------------------------

long get_file_checksum(char *file_name)
{
  int file;
  long checksum = 0;
  char *ptr;
  char *buf;
  size_t len_read;

  if ((buf = (char *) malloc(0xFFFF)) == NULL)
    {
      error("get_file_checksum", NULL, ENOMEM);
      exit(-1);
    }

  if ((file = open_file(file_name, O_RDONLY)) < 0)
    exit(-1);

  while ((len_read = read_from_file(file, buf, 0xFFFF)) > 0)
    for (ptr=buf; ptr<(buf+len_read); ++ptr)
      checksum += *ptr;

  if (len_read == -1)
    exit(-1);

  if ((close_file(file)) != 0)
    exit(-1);

  free(buf);
  return checksum;
}


//--------------------------------------------------------------------------------------------
//||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
//--------------------------------------------------------------------------------------------
